/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Permissions panel.
 * Original code from browser/components/preferences/permissions.js (Mozilla Firefox 30.0a1 source code)
 *
 * ************************************************************************
 */

"use strict";

const NOTIFICATION_FLUSH_PERMISSIONS = "flush-pending-permissions";
const PERMISSION_TYPE                = "cookie";
const COLUMN_NAME_SITE               = "treeColSite";
const COLUMN_NAME_STATUS             = "treeColCapability";
const PROPERTY_NAME_SITE             = "rawHost";
const PROPERTY_NAME_STATUS           = "capability";
const SORT_DIRECTION_ASCENDING       = true;

function Permission(host, rawHost, type, capability, capabilityId) {
  this.host         = host;
  this.rawHost      = rawHost;
  this.type         = type;
  this.capability   = capability;
  this.capabilityId = capabilityId;
}

function permissionsComparator(a, b) {
  return a.toLowerCase().localeCompare(b.toLowerCase());
}

var gCookieKeeperPermissionsPrefs = {
  _permissions       : [],
  _bundle            : null,
  _tree              : null,
  _lastSortProperty  : PROPERTY_NAME_SITE,
  _lastSortDirection : !SORT_DIRECTION_ASCENDING,

  _view: {
    _rowCount: 0,
    get rowCount() {
      return this._rowCount;
    },

    getCellText: function(aRow, aColumn) {
      if (aColumn.id == COLUMN_NAME_SITE) {
        return gCookieKeeperPermissionsPrefs._permissions[aRow].rawHost;
      } else if (aColumn.id == COLUMN_NAME_STATUS) {
        return gCookieKeeperPermissionsPrefs._permissions[aRow].capability;
      }
      return "";
    },

    isSeparator: function(aIndex) { return false; },
    isSorted: function() { return false; },
    isContainer: function(aIndex) { return false; },
    setTree: function(aTree) {},
    getImageSrc: function(aRow, aColumn) {},
    getProgressMode: function(aRow, aColumn) {},
    getCellValue: function(aRow, aColumn) {},
    cycleHeader: function(aColumn) {},
    getRowProperties: function(aRow){ return ""; },
    getColumnProperties: function(aColumn) { return ""; },
    getCellProperties: function(aRow,aColumn) {
      if (aColumn.id == COLUMN_NAME_SITE) {
        return "ltr";
      }
      return "";
    }
  },

  init: function() {
    this._bundle = document.getElementById("bundlePreferences");

    Services.obs.notifyObservers(null, NOTIFICATION_FLUSH_PERMISSIONS, PERMISSION_TYPE);
    Services.obs.addObserver(this, "perm-changed", false);

    this._loadPermissions();

    var urlField = document.getElementById("url");
    urlField.focus();
  },

  uninit: function() {
    Services.obs.removeObserver(this, "perm-changed");
  },

  _getCapabilityString: function(aCapability) {
    var stringKey = null;
    switch (aCapability) {
    case MOZ_COOKIE_PERM_ALLOW:
      stringKey = "can";
      break;
    case MOZ_COOKIE_PERM_DENY:
      stringKey = "cannot";
      break;
    case MOZ_COOKIE_PERM_FIRST:
      stringKey = "canAccessFirstParty";
      break;
    case MOZ_COOKIE_PERM_SESSION:
      stringKey = "canSession";
      break;
    }
    return this._bundle.getString(stringKey);
  },

  addPermission: function(aCapability) {

    var textbox = document.getElementById("url");
    var host = textbox.value.replace(/^\s*([-\w]*:\/+)?/, ""); // trim any leading space and scheme
    try {
      let uri = Services.io.newURI("http://" + host, null, null);
      host = uri.host;
    } catch(ex) {
      var message = this._bundle.getString("invalidURI");
      var title = this._bundle.getString("invalidURITitle");
      Services.prompt.alert(window, title, message);
      return;
    }

    // check whether the permission already exists, if not, add it
    var exists = false;
    for (let i=0; i<this._permissions.length; i++) {
      if (this._permissions[i].rawHost == host) {
        // Avoid calling the permission manager if the capability settings are
        // the same. Otherwise allow the call to the permissions manager to
        // update the listbox for us.
        exists = this._permissions[i].capabilityId == aCapability;
        break;
      }
    }

    if (!exists) {
      host = (host.charAt(0) == ".") ? host.substring(1,host.length) : host;
      let uri = Services.io.newURI("http://" + host, null, null);
      Services.perms.add(uri, PERMISSION_TYPE, aCapability);
    }
    textbox.value = "";
    textbox.focus();
    this.onHostInput(textbox);

    // enable "remove all" button as needed
    document.getElementById("removeAllPermissions").disabled = this._permissions.length == 0;
  },

  onHostInput: function(aSiteField) {
    document.getElementById("btnSession").disabled = !aSiteField.value;
    document.getElementById("btnBlock").disabled = !aSiteField.value;
    document.getElementById("btnAllow").disabled = !aSiteField.value;
  },

  onHostKeyPress: function(aEvent) {
    if (aEvent.keyCode == KeyEvent.DOM_VK_RETURN)
      document.getElementById("btnAllow").click();
  },

  observe: function(aSubject, aTopic, aData) {
    if (aTopic == "perm-changed") {
      var permission = aSubject.QueryInterface(Ci.nsIPermission);
      if (aData == "added") {
        this._addPermissionToList(permission);
        this._view._rowCount++;
        this._tree.treeBoxObject.rowCountChanged(this._view.rowCount - 1, 1);
        // Re-do the sort, since we inserted this new item at the end.
        gTreeUtils.sort(this._tree, this._view, this._permissions,
                        this._lastSortProperty,
                        permissionsComparator,
                        this._lastSortProperty,
                        !this._lastSortDirection);
      } else if (aData == "changed") {
        for (let i=0; i<this._permissions.length; i++) {
          if (this._permissions[i].host == permission.host) {
            this._permissions[i].capability   = this._getCapabilityString(permission.capability);
            this._permissions[i].capabilityId = permission.capability;
            break;
          }
        }
        // Re-do the sort, if the status changed from Block to Allow
        // or vice versa, since if we're sorted on status, we may no
        // longer be in order.
        if (this._lastSortProperty == PROPERTY_NAME_STATUS) {
          gTreeUtils.sort(this._tree, this._view, this._permissions,
                          this._lastSortProperty,
                          permissionsComparator,
                          this._lastSortProperty,
                          !this._lastSortDirection);
        }
        this._tree.treeBoxObject.invalidate();
      } else if (aData == "deleted") {
        log.d("Update deleted perm " + permission.host);
        let found = false;
        for (let i=0; i<this._permissions.length; i++) {
          if (this._permissions[i].host == permission.host) {
            this._tree.view.selection.clearSelection();
            this._permissions.splice(i, 1);
            this._view._rowCount--;
            this._tree.treeBoxObject.rowCountChanged(this._view.rowCount + 1, -1);
            this._tree.treeBoxObject.invalidate(); // force refresh
            found = true;
          }
        }
        if (!found) {
          log.e("perm not found!");
        }
      } else {
        log.d("prefs.permissions.js/observe TODO: " + aData);
      }
    }
  },

  onPermissionSelected: function() {
    var hasSelection = this._tree.view.selection.count > 0;
    var hasRows = this._tree.view.rowCount > 0;
    document.getElementById("removePermission").disabled = !hasRows || !hasSelection;
    document.getElementById("removeAllPermissions").disabled = !hasRows;
  },

  onPermissionDeleted: function() {
    if (!this._view.rowCount) {
      return;
    }
    var removedPermissions = [];
    gTreeUtils.deleteSelectedItems(this._tree, this._view, this._permissions, removedPermissions);
    for (let i=0; i<removedPermissions.length; i++) {
      var p = removedPermissions[i];
      try {
        Services.perms.remove(p.host, p.type);
      } catch (ex) {
        let uri = Services.io.newURI("http://" + p.host, null, null);
        Services.perms.remove(uri, p.type);
      }
    }
    document.getElementById("removePermission").disabled = !this._permissions.length;
    document.getElementById("removeAllPermissions").disabled = !this._permissions.length;
  },

  onAllPermissionsDeleted: function() {
    if (!this._view.rowCount) {
      return;
    }
    var removedPermissions = [];
    gTreeUtils.deleteAll(this._tree, this._view, this._permissions, removedPermissions);
    for (let i=0; i<removedPermissions.length; i++) {
      var p = removedPermissions[i];

      try {
        Services.perms.remove(p.host, p.type);
      } catch (ex) {
        let uri = Services.io.newURI("http://" + p.host, null, null);
        Services.perms.remove(uri, p.type);
      }
    }
    document.getElementById("removePermission").disabled = true;
    document.getElementById("removeAllPermissions").disabled = true;
  },

  onPermissionKeyPress: function(aEvent) {
    if (aEvent.keyCode == aEvent.DOM_VK_DELETE) {
      this.onPermissionDeleted();
    }
  },

  onPermissionSort: function(aEvent, aPropertyName) {
    function propToColName(aName) {
      if (PROPERTY_NAME_SITE == aName) {
        return COLUMN_NAME_SITE;
      }
      if (PROPERTY_NAME_STATUS == aName) {
        return COLUMN_NAME_STATUS;
      }
      throw new Error("Invalid property:" + aName);
    }

    if (aEvent != null && aEvent.button != 0) {
      return;
    }

    var columnElem = document.getElementById(propToColName(this._lastSortProperty));
    if (columnElem != null) {
      columnElem.removeAttribute("sortDirection");
    }
    this._lastSortDirection = gTreeUtils.sort(this._tree,
                                                        this._view,
                                                        this._permissions,
                                                        aPropertyName,
                                                        permissionsComparator,
                                                        this._lastSortProperty,
                                                        this._lastSortDirection);
    this._lastSortProperty = aPropertyName;
    columnElem = document.getElementById(propToColName(aPropertyName));
    if (columnElem != null) {
      columnElem.setAttribute("sortDirection", this._lastSortDirection ? "ascending" : "descending");
    }
  },

  _loadPermissions: function() {
    this._tree = document.getElementById("treePermissions");
    this._permissions = [];

    // load permissions into a table
    var enumerator = Services.perms.enumerator;
    while (enumerator.hasMoreElements()) {
      let nextPermission = enumerator.getNext().QueryInterface(Ci.nsIPermission);
      this._addPermissionToList(nextPermission);
    }

    this._view._rowCount = this._permissions.length;

    // sort and display the table
    this._tree.treeBoxObject.view = this._view;
    this.onPermissionSort(null, this._lastSortProperty);

    // disable "remove all" button if there are none
    document.getElementById("removeAllPermissions").disabled = this._permissions.length == 0;
  },

  _addPermissionToList: function(aPermission) {
    try {
      if (aPermission.type == PERMISSION_TYPE) {
        let capabilityString = this._getCapabilityString(aPermission.capability);
        let host = null;
        if (aPermission.principal) {
          let principal = aPermission.principal;
          // log.d("aPermission=", aPermission, " capabilityString=", capabilityString, " principal=", principal);
          host = principal.baseDomain;
        } else {
          host = aPermission.host;
        }
        let p = new Permission(host,
                               (host.charAt(0) == ".") ? host.substring(1,host.length) : host,
                               aPermission.type,
                               capabilityString,
                               aPermission.capability);
        this._permissions.push(p);
      }
    } catch (ex) {
      log.e("Error adding permission: ", ex);
    }
  },

  // ***********************
  // Context menu functions:

  onContextMenu: function(aEvent) {
    var seln            = this._view.selection;
    var showContextMenu = this._permissions.length > 0 && this._view.rowCount > 0 && seln.count > 0;
    if (!showContextMenu) {
      aEvent.preventDefault();
    }
    return aEvent;
  },

  _lastSelectedItems: [],
  onShowContextCookieMenu: function(aEvent) {
    var seln            = this._view.selection;
    var mpccPermAllow   = document.getElementById("mpcc-perm-allow");
    var mpccPermSession = document.getElementById("mpcc-perm-session");
    var mpccPermDeny    = document.getElementById("mpcc-perm-deny");

    this._lastSelectedItems = [];

    if (seln.currentIndex == -1 || seln.count == 0) {
      return false;
    }

    mpccPermAllow.hidden   = false;
    mpccPermSession.hidden = false;
    mpccPermDeny.hidden    = false;

    // Hide unusefull menu items
    if (seln.count > 1) {
      let nPermAllow   = 0;
      let nPermSession = 0;
      let nPermDeny    = 0;
      let nTotal       = 0;
      let rangeCount   = seln.getRangeCount();

      for (let i = 0; i < rangeCount; i++) {
        let min = {}; var max = {};
        seln.getRangeAt(i, min, max);

        for (let j = min.value; j <= max.value; j++) {
          nTotal++;
          let item = gCookieKeeperPermissionsPrefs._permissions[j];
          this._lastSelectedItems.push(item);

          switch (item.capabilityId) {
          case MOZ_COOKIE_PERM_ALLOW:
            nPermAllow++;
            break;
          case MOZ_COOKIE_PERM_SESSION:
            nPermSession++;
            break;
          case MOZ_COOKIE_PERM_DENY:
            nPermDeny++;
            break;
          }
        }
      }

      if (nTotal == nPermAllow) {
        mpccPermAllow.hidden = true;
      } else if (nTotal == nPermSession) {
        mpccPermSession.hidden = true;
      } else if (nTotal == nPermDeny) {
        mpccPermDeny.hidden = true;
      }

    } else {
      let item = gCookieKeeperPermissionsPrefs._permissions[seln.currentIndex];
      this._lastSelectedItems.push(item);

      switch (item.capabilityId) {
      case MOZ_COOKIE_PERM_ALLOW:
        mpccPermAllow.hidden = true;
        break;
      case MOZ_COOKIE_PERM_SESSION:
        mpccPermSession.hidden = true;
        break;
      case MOZ_COOKIE_PERM_DENY:
        mpccPermDeny.hidden = true;
        break;
      }
    }

    return true;
  },

  onContextMenuPermission: function(aPerm) {
    var ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);

    for (let i=0; i<this._lastSelectedItems.length; i++) {
      let item = this._lastSelectedItems[i];
      let uri = ioService.newURI("http://" + item.host, null, null);
      Services.perms.add(uri, PERMISSION_TYPE, aPerm);
    }
  },

  onContextMenuCopy: function() {
    var sClipboard = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
    var seln       = this._view.selection;
    var ci         = seln.currentIndex;
    var rawHost    = ci == -1 ? null : gCookieKeeperPermissionsPrefs._permissions[ci].rawHost;

    if (rawHost == null) {
      return;
    }
    sClipboard.copyString(rawHost);
  }
};

var gCookieKeeperPermissionsHTML = {
  _oDynTable   : null,
  _oPermManager: null,
  _eventLock   : false,

  init: function() {
    try {
      this.uninit();

      let hTable     = document.getElementById("perm_table");
      let hTBody     = document.getElementById("perm_tbody");
      let hBtnRem    = document.getElementById("perm_button_remove");
      let hBtnRemAll = document.getElementById("perm_button_remove_all");

      this._oDynTable    = new DynTable(hTable, "perm_cookies");
      this._oPermManager = new CKPermissionsManager();

      this._oPermManager.addPermChangeObserver(this.onPermChange);

      let aPerms = this._oPermManager.fetchCookiesPermissions();
      for (let i=0; i<aPerms.length; i++) {
        let oCKPerm = aPerms[i];
        let hTrElem = this.createTrElement(oCKPerm);
        hTBody.appendChild(hTrElem);
      }

      // Allow sorting
      this._oDynTable.enableSortColumn(this._compare);
      // Do initial sorting
      this._oDynTable.doSort(null);
      // Initial column resize
      this._oDynTable.resizeBodyColumns();
      // Allow selection
      this._oDynTable.setSelectListener(this.onSelectionChange);

      // Allow to select the table
      hTable.setAttribute("tabindex", "2");

      // Enable "Remove All" button
      if (aPerms.length > 0) {
        hBtnRemAll.removeAttribute("disabled");
      } else {
        hBtnRemAll.setAttribute("disabled", "disabled");
      }

      hBtnRem.addEventListener("click", this.onRemove, false);
      hBtnRemAll.addEventListener("click", this.onRemoveAll, false);
      document.getElementById("perm_input_address").focus();
    } catch (ex) {
      log.e(ex);
    }
  },

  uninit: function() {
    let hInput     = document.getElementById("perm_input_address");
    let hTable     = document.getElementById("perm_table");
    let hTBody     = document.getElementById("perm_tbody");
    let hBtnRem    = document.getElementById("perm_button_remove");
    let hBtnRemAll = document.getElementById("perm_button_remove_all");

    hInput.value = "";
    this.onInputChange(null);

    if (this._oDynTable != null) {
      this._oDynTable.destructor();
      this._oDynTable = null;
    }
    if (this._oPermManager != null) {
      this._oPermManager.destructor();
      this._oPermManager = null;
    }

    hTable.removeAttribute("tabindex");

    // Remove tbody child elements
    while (hTBody.firstElementChild) {
      let hTrElem = hTBody.firstElementChild;
      delete hTrElem.oCKPerm;
      hTBody.removeChild(hTrElem);
    }

    // Disabled buttons
    hBtnRem.setAttribute("disabled", "disabled");
    hBtnRem.removeEventListener("click", this.onRemove, false);
    hBtnRemAll.setAttribute("disabled", "disabled");
    hBtnRemAll.removeEventListener("click", this.onRemoveAll, false);

    this._eventLock = false;
  },

  createTrElement: function(oCKPerm) {
    let hTrElem = document.createElement("tr");
    hTrElem.classList.add("table_scroll_contentrow");
    hTrElem.setAttribute("contextmenu", "perm_menu_context");
    hTrElem.oCKPerm = oCKPerm;

    let hTdElem = document.createElement("td");
    hTdElem.classList.add("table_scroll_contentcell");
    hTdElem.innerHTML = oCKPerm.host;
    hTrElem.appendChild(hTdElem);

    hTdElem = document.createElement("td");
    hTdElem.classList.add("table_scroll_contentcell");
    hTdElem.innerHTML = oCKPerm.getTranslation();
    hTrElem.appendChild(hTdElem);

    return hTrElem;
  },

  onPermChange: function(oCKPerm, sChange) {
    let that = gCookieKeeperPermissionsHTML;
    if (that._eventLock) {
      return;
    }

    let hTBody    = document.getElementById("perm_tbody");
    let btnRemAll = document.getElementById("perm_button_remove_all");

    if (sChange == "added") {
      let hTrElem = that.createTrElement(oCKPerm);
      hTBody.appendChild(hTrElem);
      that._oDynTable.resizeBodyColumns();
    } else if (sChange == "changed") {
      let nElems  = hTBody.children.length;
      for (let i=0; i<nElems; i++) {
        let hTrBodyElem = hTBody.children[i];
        if (hTrBodyElem.oCKPerm.same(oCKPerm)) {
          // Update the permission text value
          hTrBodyElem.children[1].innerHTML = oCKPerm.getTranslation();
        }
      }
    } else if (sChange == "deleted") {
      let nElems  = hTBody.children.length;
      let nIdxDel = -1;

      // Find the deleted row
      for (let i=0; i<nElems; i++) {
        let hTrBodyElem = hTBody.children[i];
        if (hTrBodyElem.oCKPerm.equals(oCKPerm)) {
          nIdxDel = i;
          nElems--;
          hTBody.removeChild(hTrBodyElem);
          break;
        }
      }

      if (nIdxDel == -1) {
        log.e("Permission for " + oCKPerm.host + " not found");
      } else for (let i=nIdxDel; i<nElems; i++) {
        // Update the styles odd/even
        let hTrBodyElem = hTBody.children[i];
        if ((i % 2) == 0) {
          hTrBodyElem.classList.remove("table_scroll_contentrow_odd");
          hTrBodyElem.classList.add("table_scroll_contentrow_even");
        } else {
          hTrBodyElem.classList.remove("table_scroll_contentrow_even");
          hTrBodyElem.classList.add("table_scroll_contentrow_odd");
        }
      }
      that._oDynTable.updateSelection();
    }

    if (hTBody.children.length > 0) {
      btnRemAll.removeAttribute("disabled");
    } else {
      btnRemAll.setAttribute("disabled", "disabled");
    }
  },

  onSelectionChange: function(aAddedItems, aRemovedItems, aCurrentItems) {
    let btn = document.getElementById("perm_button_remove");

    if (aCurrentItems.length > 0) {
      btn.removeAttribute("disabled");
    } else {
      btn.setAttribute("disabled", "disabled");
    }
  },

  // Compare 2 rows (<tr> elements) of the <tbody>.
  // Use to sort the table
  _compare: function(a, b) {
    let aSortColumns = gCookieKeeperPermissionsHTML._oDynTable.aSortColumns;
    if (aSortColumns == null || !a.oCKPerm || !b.oCKPerm) {
      return 0;
    }

    let sortNormal = aSortColumns[0] > 0;
    let nCols      = aSortColumns.length;
    let aHost      = a.oCKPerm.host;
    let bHost      = b.oCKPerm.host;
    let aHTML      = a.children[1].innerHTML.trim();
    let bHTML      = b.children[1].innerHTML.trim();

    for (let i=0; i<nCols; i++) {
      let nColId = Math.abs(aSortColumns[i]);
      let cmpVal = 0;
      switch (nColId) {
      case 1:
        cmpVal = gCommon.cmpHostName(aHost, bHost);
        break;
      case 2:
        cmpVal = aHTML.toLowerCase().localeCompare(bHTML.toLowerCase());
        break;
      }

      if (cmpVal != 0) {
        return sortNormal ? cmpVal : -cmpVal;
      }
    }
    return 0;
  },

  onRemove: function(evtClick) {
    let hTBody     = document.getElementById("perm_tbody");
    let hBtnRem    = document.getElementById("perm_button_remove");
    let hBtnRemAll = document.getElementById("perm_button_remove_all");
    let that       = gCookieKeeperPermissionsHTML;
    let aSelected  = that._oDynTable.aSelected;
    let nSelected  = aSelected.length;
    let aHtrElems  = new Array(nSelected);

    if (nSelected == 0) {
      return false;
    }

    that._eventLock = true;
    that._oDynTable.resetSelection();

    for (let i=0; i<nSelected; i++) {
      aHtrElems[i] = hTBody.children[aSelected[i]];
    }
    for (let i=0; i<nSelected; i++) {
      let hTrElem = aHtrElems[i];
      that._oPermManager.remove(hTrElem.oCKPerm);
      hTBody.removeChild(hTrElem);
    }
    hBtnRem.setAttribute("disabled", "disabled");
    if (hTBody.children.length == 0) {
      hBtnRemAll.setAttribute("disabled", "disabled");
    } else for (let i=0; i<hTBody.children.length; i++) {
      // Update style odd/even to rows
      let hTrBodyElem = hTBody.children[i];
      if ((i % 2) == 0) {
        hTrBodyElem.classList.remove("table_scroll_contentrow_odd");
        hTrBodyElem.classList.add("table_scroll_contentrow_even");
      } else {
        hTrBodyElem.classList.remove("table_scroll_contentrow_even");
        hTrBodyElem.classList.add("table_scroll_contentrow_odd");
      }
    }

    that._eventLock = false;
    return false;
  },

  onRemoveAll: function(evtClick) {
    let that       = gCookieKeeperPermissionsHTML;
    let hTBody     = document.getElementById("perm_tbody");
    let hBtnRem    = document.getElementById("perm_button_remove");
    let hBtnRemAll = document.getElementById("perm_button_remove_all");
    let oPManager  = that._oPermManager;
    let nElems     = hTBody.children.length;
    let aPerms     = new Array(nElems);

    that._eventLock = true;
    that._oDynTable.resetSelection();

    for (let i=0; i<nElems; i++) {
      let hTrBodyElem = hTBody.firstElementChild;
      aPerms[i] = hTrBodyElem.oCKPerm;
      hTBody.removeChild(hTrBodyElem);
    }

    for (let i=0; i<nElems; i++) {
      let oCKPerm = aPerms[i];
      if (!oPManager.remove(oCKPerm)) {
        log.e("Error removing permission of " + oCKPerm.host);
      }
    }

    hBtnRem.setAttribute("disabled", "disabled");
    hBtnRemAll.setAttribute("disabled", "disabled");

    that._eventLock = false;
  },

  onInputChange: function(evtChange) {
    let hInput      = document.getElementById("perm_input_address");
    let hBtnBlock   = document.getElementById("perm_button_block");
    let hBtnAllow   = document.getElementById("perm_button_allow");
    let hBtnSession = document.getElementById("perm_button_session");

    if (hInput.value.length > 0) {
      hBtnBlock.removeAttribute("disabled");
      hBtnBlock.addEventListener("click", this.onAddPerm, false);

      hBtnAllow.removeAttribute("disabled");
      hBtnAllow.addEventListener("click", this.onAddPerm, false);

      hBtnSession.removeAttribute("disabled");
      hBtnSession.addEventListener("click", this.onAddPerm, false);
    } else {
      hBtnBlock.setAttribute("disabled", "disabled");
      hBtnBlock.removeEventListener("click", this.onAddPerm, false);

      hBtnAllow.setAttribute("disabled", "disabled");
      hBtnAllow.removeEventListener("click", this.onAddPerm, false);

      hBtnSession.setAttribute("disabled", "disabled");
      hBtnSession.removeEventListener("click", this.onAddPerm, false);
    }

    return true;
  },

  onAddPerm: function(evtClick) {
    let hInput     = document.getElementById("perm_input_address");
    let that       = gCookieKeeperPermissionsHTML;
    let oPManager  = that._oPermManager;
    let capability = oPManager.PERM_DEFAULT;
    let domain     = hInput.value;
    let uri        = null;

    if (domain.length == 0) {
      log.d("Input value is empty");
      return true;
    }

    if (this.id == "perm_button_block") {
      capability = oPManager.PERM_DENY;
    } else if (this.id == "perm_button_allow") {
      capability = oPManager.PERM_ALLOW;
    } else if (this.id == "perm_button_session") {
      capability = oPManager.PERM_SESSION;
    }

    try {
      try {
        uri = Services.io.newURI("http://" + domain, null, null);
      } catch (ex) {
        uri = Services.io.newURI(domain, null, null);
      }
    } catch (ex) {
      log.e("Invalid domain: " + domain);
      log.e(ex);
      return true;
    }

    if (oPManager.add(uri.host, capability) == null) {
      // FIXME: translation
      window.alert("Invalid domain: '" + uri.host + "'");
    } else {
      hInput.value = "";
      that.onInputChange(null);
    }

    return true;
  },

  onContextMenu: function(evtContextMenu) {
    let hTBody    = document.getElementById("perm_tbody");
    let item      = evtContextMenu.target.id;
    let aSelected = this._oDynTable.aSelected;

    if (item == "perm_menuitem_remove") {
      this.onRemove(evtContextMenu);
    } else if (item == "perm_menuitem_copy") {
      let sClipboard = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
      if (aSelected.length > 1) {
        let cpyText = "";
        for (let i=0; i<aSelected.length; i++) {
          let hTrElem = hTBody.children[aSelected[i]];
          cpyText += hTrElem.oCKPerm.host + "\n";
        }
        sClipboard.copyString(cpyText);
      } else if (this._oDynTable.nIdxLastSelected != -1) {
        let hTrElem = hTBody.children[this._oDynTable.nIdxLastSelected];
        sClipboard.copyString(hTrElem.oCKPerm.host);
      }

    } else {
      // Change permission value
      let capability = this._oPermManager.PERM_DEFAULT;

      if (item == "perm_menuitem_allow") {
        capability = this._oPermManager.PERM_ALLOW;
      } else if (item == "perm_menuitem_block") {
        capability = this._oPermManager.PERM_DENY;
      } else if (item == "perm_menuitem_session") {
        capability = this._oPermManager.PERM_SESSION;
      }

      for (let i=0; i<aSelected.length; i++) {
        let hTrElem = hTBody.children[aSelected[i]];
        this._oPermManager.add(hTrElem.oCKPerm.host, capability);
      }
    }

    return false;
  }
};
